"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.init = void 0;
var tslib_1 = require("tslib");
var types_1 = require("./types");
var constants_1 = require("./constants");
var prepare_options_1 = require("./prepare_options");
var cross_fetch_1 = require("cross-fetch");
var exponential_backoff_1 = require("exponential-backoff");
require("@github1/amfjs/amf");
function createFetch(opts, allConfigs, abortController) {
    var fetchOpts = (0, prepare_options_1.default)(opts);
    (0, prepare_options_1.prepareUrl)(fetchOpts);
    (0, prepare_options_1.prepareCredentials)(fetchOpts);
    var abortSignal = abortController.signal;
    return (0, cross_fetch_1.default)(fetchOpts.url, tslib_1.__assign(tslib_1.__assign({}, fetchOpts), { signal: abortSignal })).then(function (fetchResponse) {
        var responseContentType = fetchResponse.headers.get(constants_1.default.content_type) || '';
        var resPromise;
        if (responseContentType.indexOf(constants_1.default.application_json) > -1) {
            resPromise = fetchResponse.json().then(wrapResponse(fetchResponse));
        }
        else if (responseContentType.indexOf(constants_1.default.application_amf) > -1) {
            resPromise = fetchResponse
                .arrayBuffer()
                .then(function (buffer) {
                var deserialized = new amf.Deserializer(new Uint8Array(buffer)).readObject();
                return wrapResponse(fetchResponse)(deserialized);
            })
                .catch(function (err) {
                if (typeof err === 'string') {
                    err = new Error(err);
                }
                throw err;
            });
        }
        else {
            resPromise = fetchResponse.text().then(wrapResponse(fetchResponse));
        }
        if (fetchResponse.status < 200 || fetchResponse.status >= 400) {
            return resPromise.then(function (resp) {
                throw new types_1.AjaxServiceError(resp);
            });
        }
        return resPromise;
    });
}
var ajaxServiceInit = function (configs) {
    if (configs === void 0) { configs = []; }
    var ajaxServiceInstance = {
        constants: constants_1.default,
        get: function (opts) {
            return this.send(tslib_1.__assign(tslib_1.__assign({}, opts), { method: 'GET' }));
        },
        post: function (opts) {
            return this.send(tslib_1.__assign(tslib_1.__assign({}, opts), { method: 'POST' }));
        },
        delete: function (opts) {
            return this.send(tslib_1.__assign(tslib_1.__assign({}, opts), { method: 'DELETE' }));
        },
        send: function (opts) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            function fetchCreator() {
                var currentOptsRef = [];
                var abortController = new AbortController();
                abortController.signal.addEventListener('abort', function () {
                    allConfigs
                        .filter(function (config) { return !!(config === null || config === void 0 ? void 0 : config.onCancel); })
                        .map(function (config) { return config.onCancel; })
                        .forEach(function (onCancel) {
                        onCancel({
                            req: currentOptsRef[0],
                        });
                    });
                });
                var cancelFunc = function () {
                    try {
                        allConfigs
                            .filter(function (config) { return !!(config === null || config === void 0 ? void 0 : config.onCancel); })
                            .map(function (config) { return config.onCancel; })
                            .forEach(function (onCancel) {
                            onCancel({
                                req: currentOptsRef[0],
                            });
                        });
                        var cancelError = new Error('cancelled');
                        cancelError.req = currentOptsRef[0];
                        return Promise.reject(cancelError);
                    }
                    finally {
                        abortController.abort();
                    }
                };
                var mainFetchCreator = function (o) {
                    return createFetch(o, allConfigs, abortController).then(function (res) {
                        return Promise.all(allConfigs
                            .filter(function (config) { return !!(config === null || config === void 0 ? void 0 : config.onResponse); })
                            .map(function (config) { return config.onResponse; })
                            .reduce(function (a, b) {
                            return tslib_1.__spreadArray(tslib_1.__spreadArray([], a, true), [b({ req: o, res: res })], false).filter(function (item) { return !!item; });
                        }, [])).then(function () { return res; });
                    });
                };
                var interceptors = allConfigs
                    .filter(function (config) { return !!(config === null || config === void 0 ? void 0 : config.onRequest); })
                    .map(function (config) { return config.onRequest; });
                var index = 0;
                var reqState = {
                    cancel: cancelFunc,
                    next: null,
                    retryState: retryState,
                };
                var theNextInterceptor = function (o) {
                    return (interceptors[index++] || mainFetchCreator)(o, tslib_1.__assign(tslib_1.__assign({}, reqState), { next: function (nReq) {
                            currentOptsRef[0] = nReq;
                            return theNextInterceptor(nReq, tslib_1.__assign({}, reqState));
                        } }));
                };
                return theNextInterceptor(opts, reqState);
            }
            var retryState, allConfigs, numOfAttempts;
            var _a;
            return tslib_1.__generator(this, function (_b) {
                retryState = {
                    attemptNumber: 0,
                    numOfAttempts: 0,
                    err: undefined,
                };
                allConfigs = tslib_1.__spreadArray(tslib_1.__spreadArray([], (configs || []), true), ((opts === null || opts === void 0 ? void 0 : opts.configs) || []), true);
                numOfAttempts = ((_a = opts === null || opts === void 0 ? void 0 : opts.retry) === null || _a === void 0 ? void 0 : _a.attempts) || 10;
                return [2, (0, exponential_backoff_1.backOff)(fetchCreator, {
                        numOfAttempts: numOfAttempts,
                        retry: function (e, attemptNumber) {
                            var _a;
                            e.retryAttemptNumber = attemptNumber;
                            retryState.attemptNumber = attemptNumber;
                            retryState.numOfAttempts = numOfAttempts;
                            retryState.err = e;
                            return (((_a = e === null || e === void 0 ? void 0 : e.response) === null || _a === void 0 ? void 0 : _a.status) === 503 ||
                                /network request failed/i.test(e.message) ||
                                /^request to.*failed, reason:/i.test(e.message) ||
                                /^failed to fetch$/i.test(e.message));
                        },
                    })];
            });
        }); },
    };
    return ajaxServiceInstance;
};
ajaxServiceInit.constants = constants_1.default;
exports.init = ajaxServiceInit;
function wrapResponse(fetchResponse) {
    return function (body) {
        return {
            status: fetchResponse.status,
            data: body,
            headers: convertHeaders(fetchResponse.headers),
        };
    };
}
function convertHeaders(headers) {
    var obj = {};
    headers.forEach(function (v, k) {
        obj[k] = v;
    });
    return obj;
}
//# sourceMappingURL=index.js.map