"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.prepareCredentials = exports.prepareUrl = void 0;
var tslib_1 = require("tslib");
require("@github1/amfjs/amf");
var constants_1 = require("./constants");
var windowOrGlobal = (typeof self === 'object' && self.self === self && self) ||
    (typeof global === 'object' && global.global === global && global) ||
    this;
var resolveOrigin = function () {
    return windowOrGlobal && windowOrGlobal.location
        ? "".concat(windowOrGlobal.location.protocol, "//").concat(windowOrGlobal.location.hostname).concat(windowOrGlobal.location.port ? ':' + windowOrGlobal.location.port : '')
        : '';
};
function normalizeURL(url) {
    var match = /([a-z0-9]+:\/\/)(.*)$/i.exec(url);
    var protocol = match[1];
    var rest = match[2];
    return "".concat(protocol.toLowerCase()).concat(rest
        .replace(/\/+/g, '/')
        .replace(/(^\/+|\/+$)/g, '')
        .replace(/\/+\?/, '?'));
}
var prepareUrl = function (opts) {
    var resolvedOrigin = opts.origin || resolveOrigin();
    opts.url = /^[a-z0-9]+:\/\//i.test(opts.url)
        ? normalizeURL(opts.url)
        : normalizeURL("".concat(resolvedOrigin, "/").concat(opts.url));
    return opts;
};
exports.prepareUrl = prepareUrl;
var regPrivNet = /^(localhost$|127\.|192\.168|10\.)([0-9.]+)?$/;
var prepareCredentials = function (opts) {
    if (!opts.credentials) {
        var resolvedOrigin = opts.origin || resolveOrigin();
        opts.credentials = 'same-origin';
        var urlHostname = extractHostname(opts.url);
        if (regPrivNet.test(urlHostname)) {
            opts.credentials = 'include';
        }
        else if (resolvedOrigin) {
            if (stripSubdomain(extractHostname(resolvedOrigin)) ===
                stripSubdomain(urlHostname)) {
                opts.credentials = 'include';
            }
        }
    }
    return opts;
};
exports.prepareCredentials = prepareCredentials;
exports.default = (function (opts) {
    var _a;
    var resolvedOrigin = opts.origin || resolveOrigin();
    opts.headers = opts.headers || {};
    var fetchOpts = {
        url: opts.url,
        method: opts['method'],
        body: opts['data'],
        credentials: opts.credentials,
        headers: tslib_1.__assign(tslib_1.__assign({}, opts['headers']), (_a = {}, _a[constants_1.default.content_type] = opts.headers[constants_1.default.content_type] ||
            opts[constants_1.default.content_type] ||
            opts['contentType'] ||
            constants_1.default.application_json, _a[constants_1.default.accept] = opts.headers[constants_1.default.accept] ||
            opts[constants_1.default.accept] ||
            constants_1.default.application_json, _a)),
    };
    var hasBodyData = fetchOpts.body && typeof fetchOpts.body === 'object';
    fetchOpts.headers[constants_1.default.request_origin] = resolvedOrigin;
    switch (fetchOpts.headers[constants_1.default.content_type]) {
        case constants_1.default.application_amf: {
            var serializer = new amf.Serializer();
            serializer.writeObject(fetchOpts.body);
            fetchOpts.body = new Uint8Array(serializer.writer.data);
            fetchOpts.method = 'POST';
            break;
        }
        case constants_1.default.application_json: {
            if (hasBodyData) {
                fetchOpts.body = JSON.stringify(fetchOpts.body);
            }
            break;
        }
    }
    return fetchOpts;
});
var regHostname = /^(https?:\/\/)([^:\/]+)/i;
var extractHostname = function (url) {
    var result = regHostname.exec(url);
    return result === null || result.length < 3 ? '_' : result[2];
};
var stripSubdomain = function (hostname) {
    var parts = hostname.split('.');
    while (parts.length > 2) {
        parts.shift();
    }
    return parts.join('.');
};
//# sourceMappingURL=prepare_options.js.map